/**
 * HPCU Bi-Weekly Challenge: Parallel Ideal Gas 2
 * Main file - runs a simulation of particles moving randomly in and between
 * two flasks connected by one stopcock per flask.
 *
 * @author Aaron Weeden, Shodor Education Foundation
 * @version 5.0
 * @since 1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "constants.h"
#include "typedefs.h"
#include "initialization.h"
#include "simulation.h"
#include "memory-management.h"

/**
 * Runs the program.
 *
 * @return 0 on success, non-zero on failure
 */
int main() {

  /* Declare constants whose values depend on other constants */
  const double V1 = FLASK_1_ROW_COUNT * FLASK_1_COLUMN_COUNT + 1.0;
  const double V2 = FLASK_2_ROW_COUNT * FLASK_2_COLUMN_COUNT + 1.0;
  const double VRATIO = V1 / V2;

  /* Declare model data */
  flask_type flask1, flask2;
  particle_list_type particleList;
  double ratio;

  /* Seed random number generator */
  srandom(0/*time(NULL)*/);

  /* Set up flasks and particles */
  initializeFlask(&flask1, 1, FLASK_1_ROW_COUNT, FLASK_1_COLUMN_COUNT,
    FLASK_1_STOPCOCK_ROW);
  initializeFlask(&flask2, 2, FLASK_2_ROW_COUNT, FLASK_2_COLUMN_COUNT,
    FLASK_2_STOPCOCK_ROW);
  initializeParticles(&particleList, &flask1, PARTICLE_BLOCK_ROW_COUNT,
    PARTICLE_BLOCK_COLUMN_COUNT, PARTICLE_BLOCK_TOP_ROW,
    PARTICLE_BLOCK_LEFT_COLUMN);

  /* Run the simulation */
  ratio = simulate(&flask1, &flask2, &particleList, TIME_COUNT, VRATIO);

  /* Print the final ratio */
  printf("Final ratio = %f\n", ratio);

  /* Release allocated memory */
  free(particleList.elements);
  freeFlask(&flask2);
  freeFlask(&flask1);

  return 0;

}
